<?php

App::uses('EvShopAppModel', 'EvShop.Model');

class ProductVariant extends EvShopAppModel {

	public $belongsTo = array(
		"Product" => array(
			'className' => 'EvShop.Product'
		)
	);

	public $hasMany = array(
		'ProductAttributeOptionProductVariant' => array(
			'className' => 'EvShop.ProductAttributeOptionProductVariant'
		)
	);

	public $hasAndBelongsToMany = array(

		"ProductAttributeOption" => array(
			'className' => 'EvShop.ProductAttributeOption', 
			'joinTable' => 'ev_shop_product_attribute_options_product_variants'
		)

	);


/**
 * Generates a unique variant product key using the given options 
 * @param array $variantOptions An array of the variant options in the form array(array('id' => '..', 'name' => '..'), array('id' => '..', 'name' => '..'))
 * @return string An ordered list of the option ids ordered and joined by "-" e.g. "3-5-9-21"
 */
	public function generateKeyFromVariantOptions($variantOptions) {

		$variantOptionIds = Hash::extract($variantOptions, '{n}.id');

		if(!empty($variantOptionIds)) {
			
			sort($variantOptionIds);
			return implode("-", $variantOptionIds);

		} else {

			return "no-attributes";

		}


	}

/**
 * Returns a list of variants for the given product indexed by the product option ids 
 * @see ProductVariant::generateKeyFromVariantOptions()
 * @param   $productId 
 * @return array An array of product variants indexed using ProductVariant::generateKeyFromVariantOptions() in a find('all') format
 */
	public function getProductVariantsIndexedByAttributeOptions($productId) {

		//Get all product variants for the given product and their product attribute options

		$variants = $this->find('all', array(
				'conditions' => array(
					'ProductVariant.product_id' => $productId
				), 
				'contain' => array(
					'ProductAttributeOption' => array(
						'order' => 'ProductAttributeOption.product_attribute_id'
					)
				)
			)
		);

		$indexedArray = array();

		foreach($variants as $variant) {

			$variantArray = $variant['ProductVariant'];
			unset($variant['ProductVariant']);

			$variantArray = array_merge($variantArray, $variant);

			$indexedArray[$this->generateKeyFromVariantOptions($variant['ProductAttributeOption'])] = $variantArray;
		}

		return $indexedArray;

	}


/**
 * Returns a find('all') array with all the available product variants for the given product
 * Available is defined by the following criteria
 *	 1. The variant has all of the attributes that are currently attached to the product 
 *	 2. The variant has a price which is not empty
 * 
 * @param $productId (int) - Id of the product you wish to fetch variants for
 * @param $frontEnd If set to true will only bring out those available for purchase (i.e. price of )
 * @param $query (array) - Any additional parameters you wish to pass into the query
 * @return (array) 	find('all') format array of Product Variants 
 */
	public function getAvailableProductVariants($productId, $query = array()) {

		$findQuery = array_merge($query, array(
			'conditions' =>	array(
				'ProductVariant.product_id' => $productId, 
				'ProductVariant.price <>' => ''
			), 
			'contain' => array(
				'ProductAttributeOption' => array(
					'ProductAttribute'
				)
			)
		));

		$productVariants = $this->find('all', $findQuery);

		return $productVariants;

	}


/**
 * Removes any product variants that are no longer applicable with the current combinations
 * of attributes. This happens when the variants of a product are changed.
 * @todo Rewrite into a single SQL query
 */
	public function cleanUpRedundantVariants($productId) {

		//First get all the product variants which dont have the same number of attributes as they should
		//But the attrbutes dont line up 
		$invalidVariants = $this->ProductAttributeOptionProductVariant->find('all', array(
				'fields' => 'product_variant_id',
				'conditions' => array(
					'product_attribute_option_id NOT IN (
						SELECT product_attribute_option_id 
						FROM ev_shop_product_attribute_options_products 
						WHERE ev_shop_product_attribute_options_products.product_id = "'.$productId.'"
					)'
				),
				'group' => 'product_variant_id'
			)
		);

		// debug($this->getLastQuery());

		//Secondly get all the product variants where they have the incorrect number of attributes
		$invalidVariants2 = $this->find('all', array(
			'joins' => array(
				array(
					'table' => 'ev_shop_product_attribute_options_product_variants',
					'alias' => 'ProductAttributeOptionProductVariant',
					'type' => 'LEFT',
					'conditions' => array(
						'ProductAttributeOptionProductVariant.product_variant_id = ProductVariant.id'
					)
				),
				array(
					'table' => 'ev_shop_product_attribute_options',
					'alias' => 'ProductAttributeOption',
					'type' => 'LEFT',
					'conditions' => array(
						'ProductAttributeOption.id = ProductAttributeOptionProductVariant.product_attribute_option_id'
					)
				)
			),
			'conditions' => array(
				'ProductVariant.product_id' => $productId
			),
			'group' => 'ProductVariant.id HAVING COUNT(DISTINCT(product_attribute_id)) <> (SELECT COUNT(DISTINCT product_attribute_id) FROM ev_shop_product_attribute_options AS ProductAttributeOption RIGHT JOIN ev_shop_product_attribute_options_products AS ProductProductAttributeOption ON ProductProductAttributeOption.product_attribute_option_id = ProductAttributeOption.id WHERE product_id = "'.$productId.'" GROUP BY product_id)'
			
		));


		//Extract the IDs
		$invalidVariantIds = Hash::extract($invalidVariants, '{n}.ProductAttributeOptionProductVariant.product_variant_id');
		$invalidVariant2Ids = Hash::extract($invalidVariants2, '{n}.ProductVariant.id');

		// debug($this->getLastQuery());
		
		//And delete
		$this->deleteAll(array(
			'ProductVariant.id' => $invalidVariantIds, 
			'ProductVariant.product_id' => $productId
		));
		$this->deleteAll(array(
			'ProductVariant.id' => $invalidVariant2Ids, 
			'ProductVariant.product_id' => $productId
		));


	}

}